;+
; NAME:
;         CMCIRSED
; PURPOSE:
;         Fit an infrared (8-1000um) spectral energy distribution to
;         a galaxy's infrared data points
; EXPLANATION:
;         Inputs at least three data points in the FIR and outputs a
;         continuous SED in the infrared comprised of a modified black
;         body SED and a mid-infrared powerlaw.  The functional form
;         of the SED fit is described in Casey et al 2012, MNRAS
;         submitted.  The fit has anywhere from two to five free
;         parameters.
;
; CALLING SEQUENCE:
;      sedflux = cmcirsed( datawave, dataflux, redshift,
;      fluxerr=fluxerr, [sedwave=sedwave, tdust=tdust ...])
;
; INPUT:
;      datawave - a 1D array of the observed wavelengths for the
;                 photometric data points (in microns)
;      dataflux - a 1D array of the flux densities of the photometric
;                 data points at wavelengths datawave (in mJy)
;      redshift - the galaxy's redshift (converts observed to
;                 rest-frame)
;      fluxerr - (keyword) the uncertainty on dataflux (in mJy) and
;                *must* be provided
;
; OUTPUT:
;      sedflux - the flux array for the best-fit SED in units of
;                log(flux density in mJy), matched to sedwave array
;      sedwave - (optional keyword) outputs the wavelength array of
;                the SED fit in units of log(wavelength in microns,
;                rest-frame
;      lfir - (optional keyword) 8-1000um luminosity for source based
;             on best-fit SED
;      tdust -(optional keyword) dust temperature for source, measured
;      from the peak(in K)
;      tdust_fit -(optional keyword) dust temperature for source,
;      best-fit (in K)
;
; EXAMPLES:
;   Take UGC02369 at 02:54:01.8 +14:58:25 and z=0.032.  It has the
;   following photometry in the infrared (taken from U et al. 2012):
;           8um     0.149+-0.004Jy
;          12um     0.230+-0.022Jy
;          24um     1.160+-0.058Jy
;          25um     1.880+-0.042Jy
;          60um     8.070+-0.043Jy
;          70um     7.823+-1.173Jy
;         100um     11.18+-0.190Jy
;         160um     8.717+-2.615Jy
;         450um     0.523+-0.120Jy
;         850um     0.072+-0.013Jy
;   To run cmcirsed, the datawave, dataflux, and datafluxerr need to
;   be consolidated:
;     IDL> datawave = [8.,12,24,25,60,70,100,160,450,850]
;     IDL> dataflux =
;     [0.149,0.230,1.160,1.880,8.070,7.823,11.18,8.717,0.523,0.072]*1e3
;     IDL> ;note that units of dataflux are in mJy
;     IDL> datafluxerr =
;     [0.004,0.022,0.058,0.042,0.043,1.173,0.190,2.615,0.120,0.013]*1e3
;     IDL> redshift = 0.032
;   Then to generate an SED fit using only two free parameters (LFIR
;   and Tdust) the command would be:
;     IDL> sedflux =
;     cmcirsed(datawave,dataflux,redshift,fluxerr=datafluxerr, $
;              /fixbeta,/fixalpha,/generalopacity, $
;              sedwave=sedwave, tdust=tdust, lfir=lfir)
;   If, however, the user wants to fit beta and alpha as free
;   parameters, then:
;     IDL> sedflux =
;     cmcirsed(datawave,dataflux,redshift,fluxerr=datafluxxerr, $
;              beta=beta,alpha=alpha,/generalopacity, $
;              sedwave=sedwave, tdust=tdust, lfir=lfir)
;   Then the output can be plotted up to verify:
;     IDL> plot, sedwave, sedflux
;     IDL> oplot, alog10(datawave/(1+redshift)), alog10(dataflux),
;     psym=1
;   And the main fit parameters are stored in the optional keywords:
;   LFIR (in log of solar luminosities), Tdust (in K), Mdust (in log
;   of solar masses)
;
; REVISION HISTORY:
;   Written by Caitlin M. Casey, 3 February 2011, IfA Hawaii
;   Revised for 'release' 9 March 2012, IfA Hawaii
;   Contact: cmcasey@ifa.hawaii.edu (or permanent address:
;   cmcasey.astro@gmail.com)


FUNCTION cmcirsed, $
   DATAWAVE, DATAFLUX, REDSHIFT, FLUXERR=FLUXERR, $ ;required information
   FIXBETA=FIXBETA, FIXALPHA=FIXALPHA, RADIO=RADIO, $ ;optional inputs
   SEDWAVE=SEDWAVE, TDUST=TDUST, TD_FIT=TD_FIT, MDUST=MDUST, LFIR=LFIR, SFBB=SFBB, SFPL=SFPL, $ ;optional outputs
   BETA=BETA, ALPHA=ALPHA, TDERR=TDERR, MDERR=MDERR, LERR=LERR, BERR=BERR, AERR=AERR, RESIDUALS=RESIDUALS, $  ;optional outputs
   GENERALOPACITY=GENERALOPACITY, INTERACTIVE=INTERACTIVE ;optional tags


if n_params() eq 0 then begin
   print, "CMCIRSED.PRO - fit a coupled greybody+powerlaw to FIR data"
   print, " SEDFLUX = CMCIRSED( DATAWAVE, DATAFLUX, REDSHIFT, [FLUXERR=FLUXERR, $ "
   print, "                      FIXBETA=FIXBETA, FIXALPHA=FIXALPHA, $  ;(optional inputs) "
   print, "                      SEDWAVE=SEDWAVE, TDUST=TDUST, TD_FIT=TD_FIT, MDUST=MDUST, LFIR=LFIR, BETA=BETA, $ ;(optional outputs) "
   print, "                      ALPHA=ALPHA, TDERR=TDERR, MDERR=MDERR, LERR=LERR, BERR=BERR, SFBB=SFBB, RESIDUALS=RESIDUALS, $ ;(optional outputs) "
   print, "                      /GENERALOPACITY, /INTERACTIVE, /RADIO) ;(optional tags)"
   print, "                         input units: datawave - wavelength array in um (observed)"
   print, "                                      dataflux - flux density array in mJy (observed)"
   print, "                         output units: sedflux - log(flux density in mJy) (rest-frame)"
   print, "                                       sedwave - log(wavelength in um) (rest-frame)"
   print, "                                       sfbb - sedflux for just the cold-dust blackbody component log(flux density in mJy)"
   print, "                                       tdust - dust temperature in K"
   print, "                                       lfir - log(Lfir) in solar luminosities"
   print, "                                       beta - emissivity"
   print, "                                       mdust - dust mass in log solar masses (from extrapolated 850um flux density)"
   print, "                         optional tags: fixbeta=betavalue - fix the emissivity to e.g. 1.5 or 2.0"
   print, "                                        fixalpha=alphavalue - fix the slope of the mid-IR powerlaw, alpha to e.g. 2.1-2.5"
   print, "                                        /radio - add a radio synchrotron component, assumes FIR/radio correlation and scales to LFIR and slope of 0.8"
   print, "                                        /generalopacity - don't assume an optically thin BB"
   print, "                                        /interactive - plots up data and prompts user to select only certain FIR data points"
   print, "       ** written by Caitlin M. Casey, February 2011 ** "
   return, -1
endif

; DEFINE SED VARIABLES:
; --- WAVELENGTH ---
wavearr = 10^makehx(0.,5.4,0.001)
; in units of microns
; defining a wavelength array which runs from ~3.16um to
; 251189um~2.5cm rest-frame

; ANALYZE DATA FOR COMPLETENESS:
; --- DOES AN ERROR ARRAY NOT EXIST? ---
if keyword_set(fluxerr) eq 0 then begin
; set some realistic flux errors considering the wavelength regimes
; and typical telescopes used for the observations (i.e. confusion noise)...
   fluxerr= dataflux*0.
   tmp = where(datawave le 120.,ct)
   if ct gt 0 then fluxerr[tmp] = 1.
   tmp = where(datawave gt 120. and datawave le 360.,ct)
   if ct gt 0 then fluxerr[tmp] = 5.
   tmp = where(datawave gt 360. and datawave le 510.,ct)
   if ct gt 0 then fluxerr[tmp] = 9.
   tmp = where(datawave gt 510. and datawave le 890.,ct)
   if ct gt 0 then fluxerr[tmp] = 2.
   tmp = where(datawave gt 890.,ct)
   if ct gt 0 then fluxerr[tmp] = 1.
;   tmp = where(fluxerr_alternate-fluxerr gt 0)
;   fluxerr[tmp] = fluxerr_alternate[tmp]
   print, "***** WARNING - FLUXERR ARRAY NOT SPECIFIED, USING SOME FAKE DEFAULT VALUES!! *****" 
endif
gooddata = where(finite(dataflux) and finite(fluxerr))
datawave = datawave[gooddata]
dataflux = dataflux[gooddata]
fluxerr = fluxerr[gooddata]

ndata = n_elements(datawave)
xarr = datawave/(1.+redshift) ;xarr = rest wavelength
yarr = alog10(dataflux>0.001) ;yarr = log dataflux
yerr = yarr*0. ;yerr will be the uncertainty in yarr
;CHECK FOR NON-DETECTIONS *AND* COMPUTE LOG-ERR ARRAY=YERR:
for i=0, ndata-1, 1 do begin
   snr = dataflux[i]/fluxerr[i] 
   if snr le 1 then begin
      yerr[i] = alog10(fluxerr[i]+dataflux[i]>0.01)-alog10(dataflux[i]>0.01)
   endif else begin
      yerr[i] = 0.5*(alog10(fluxerr[i]+dataflux[i])-alog10(dataflux[i]-fluxerr[i]))
   endelse
endfor
xarrall=xarr
yarrall=yarr
yerrall=yerr


longwave = where(xarr ge 40.,ctl)
shortwave = where(xarr lt 50. and xarr ge 8.,cts)
if cts ge 1 then begin
   xshort = xarr[shortwave]
   yshort = yarr[shortwave]
   yshorterr = yerr[shortwave]
;;COMPARISON - COMPUTE OFFSET IN 8-25um RANGE
;   restore,'/Users/cmcasey/idl/basic/smg_sed_Pope08.sav'
;   smgwave = alog10(wave)
;   smgflux = alog10(lnu) - 25.
;   tmp = min(abs(smgwave - alog10(xshort[0])),imin)
;   shortoffset = yshort[0]-smgflux[imin]
endif

;MAKE SURE ALL DATA IS FINITE:
tmp = where(finite(yarr) and finite(yerr),ct)
if ct gt 0 then begin
   xarr = xarr[tmp]
   yarr = yarr[tmp]
   yerr = yerr[tmp]
endif else begin
   print, "FAIL - non finite data (e.g. data= NaN, Inf, or -Inf) - rerun with finite data flux points..."
   return, -1
endelse


;--------------------------------
; COMPUTE THE BEST FIT FIR SED:
;--------------------------------

fitcase = ''
if keyword_set(generalopacity) then fitcase='generalopacity' else fitcase='otthin'

shortflag = cts
if shortflag eq 1 then begin
   snrshort = 10^(yshort)/(10^(yshort+yshorterr)-10^(yshort))
   if snrshort le 5. then shortflag = 0.5
endif
longflag = n_elements(longwave)

; DESCRIPTION OF FIT PARAMETERS -->
      ; the coefficients of the fit are as follows:
      ; [norm1, tdust, beta, (norm2), alpha, (waveturn)]
      ; norm1 is the normalization of the SED, proportional to LIR
      ; tdust is the dust temperature of the system,
      ;   representing the emission from the greybody
      ;   constrain dust temperature to 10<tdust<170K
      ; beta is the emissivity of the greybody
      ; norm2 is the normalization of the powerlaw component
      ;   and it is fixed to scale with norm1 and (tdust,alpha)
      ; alpha is the slope of the powerlaw component
      ; waveturn is the wavelength where the powerlaw turns over
      ;   and the derivative of the greybody = alpha, 
      ;   it is fixed to scale with (tdust,alpha)

; OPTICALLY THIN GREYBODY FIT + POWERLAW
if fitcase eq 'otthin' then begin
      parinfo = replicate({value:0.D, fixed:0, tied:'', limited:[0,0],limits:[0.D,0]}, 6)
      parinfo[1].limited[0] = 1
      parinfo[1].limited[1] = 1
      parinfo[1].limits[0] = 10.
      parinfo[1].limits[1] = 170.
      ; fix beta if fixbeta is set
      if keyword_set(fixbeta) ne 0 then begin
         parinfo[2].fixed = 1
         betavalue = fixbeta
      endif else begin
         parinfo[2].limited[0] = 1
         parinfo[2].limited[1] = 1
         parinfo[2].limits[0] = 0.5
         parinfo[2].limits[1] = 3.0
         betavalue = 1.5
      endelse
     ; tie the normalization of the powerlaw to the normalization of the blackbody and dust temperature
      parinfo[3].tied = 'p[0] + (3 + p[2])*alog10(p[1] * (0.000310933 + 6.99836e-05*p[4])) - alog10(exp(14393.9 * (0.000310933 + 6.99836e-05 * p[4] ))-1)'
     ; tie the turnover wavelength to the blackbody dust temperature
      parinfo[5].tied = '(1./p[1]) * (0.000310933 + 6.99836e-05*p[4])^(-1.)'
      if keyword_set(fixalpha) ne 0 then begin
         parinfo[4].fixed = 1
         alphavalue = fixalpha
      endif else begin
         parinfo[4].limited[0] = 1
         parinfo[4].limited[1] = 1
         parinfo[4].limits[0] = 0.5
         parinfo[4].limits[1] = 3.0
         alphavalue = 2.0
         if shortflag lt 1 then begin 
            parinfo[4].fixed = 1
         endif
      endelse
      p0 = 4.0        ;best starting guess for N_bb
      p1 = 30         ;best starting guess for Tdust
      p2 = betavalue  ;best starting guess for beta
      p4 = alphavalue ;best starting guess for alpha
      npl = p0 + (3 + p2)*alog10(p1 * (0.000310933 + 6.99836e-05*p4)) - alog10(exp(14393.9 * (0.000310933 + 6.99836e-05 * p4 ))-1)
      waveturn = (p1 * (0.000310933 + 6.99836e-05*p4))^(-1.)
      parinfo[*].value = [p0, p1, betavalue, npl, alphavalue, waveturn]
      p0 = parinfo.value
;      expr ='7.+4.+alog10( (10^P[0])*(x^(-(3.+p[2]))/(exp(14393.9/(x*P[1]))-1.)) + (10^P[3]) * exp(-(x/(p[5]*2./3.))^2.) * (x/p[5])^p[4] ) '
      expr ='7.+4.+alog10( (10^P[0])*(x^(-(3.+p[2]))/(exp(14393.9/(x*P[1]))-1.)) + (10^P[3]) * exp(-(x/(p[5]*3./4.))^2.) * (x/p[5])^(p[4]) ) '
      P = mpfitexpr(expr,xarr,yarr,yerr,p0,perror=dp,parinfo=parinfo)
      fluxarr =7.+4.+alog10( (10^P[0])*(wavearr^(-(3.+p[2]))/(exp(14393.9/(wavearr*P[1]))-1.)) + 10^p[3]*exp(-(wavearr/(p[5]*3./4.))^2.) * (wavearr/p[5])^(p[4]))
      fluxarr_bb =7.+4. + alog10( (10^P[0])*(wavearr^(-(3.+p[2]))/(exp(14393.9/(wavearr*P[1]))-1.)) )
      fluxarr_pl =7.+4. + alog10( (10^p[3])*(exp(-(wavearr/(p[5]*3./4.))^2.)) * (wavearr/p[5])^(p[4]) )
   endif

; GENERALOPACITY GREYBODY FIT + POWERLAW
if fitcase eq 'generalopacity' then begin
      parinfo = replicate({value:0.D, fixed:0, tied:'', limited:[0,0],limits:[0.D,0]}, 6)
      ; [norm1, tdust, beta, (norm2), alpha, (waveturn)]
      parinfo[1].limited[0] = 1
      parinfo[1].limited[1] = 1
      parinfo[1].limits[0] = 10.
      parinfo[1].limits[1] = 170.
      ; fix beta if fixbeta is set
      if keyword_set(fixbeta) ne 0 then begin
         parinfo[2].fixed = 1
         betavalue = fixbeta
      endif else begin
         parinfo[2].limited[0] = 1
         parinfo[2].limited[1] = 1
         parinfo[2].limits[0] = 0.5
         parinfo[2].limits[1] = 3.0
         betavalue = 1.5
      endelse
     ; tie the normalization of the powerlaw to the normalization of the blackbody and dust temperature
      parinfo[3].tied = 'p[0] + alog10(1-exp(-(200.*((26.6764 + 6.24629 * p[4])^(-2.) + (0.000190530 + 7.24277e-05 * p[4])*p[1]))^p[2])) + 3.*alog10((26.6764 + 6.24629 * p[4])^(-2.) + (0.000190530 + 7.24277e-05 * p[4])*p[1]) - alog10(exp(14393.9*((26.6764 + 6.24629 * p[4])^(-2.) + (0.000190530 + 7.24277e-05 * p[4])*p[1])/(p[1]))-1.)'
     ; tie the turnover wavelength to the blackbody dust temperature
      parinfo[5].tied = '((26.6764 + 6.24629 * p[4])^(-2.) + (0.000190530 + 7.24277e-05 * p[4])*p[1])^(-1.)'
      if keyword_set(fixalpha) ne 0 then begin
         parinfo[4].fixed = 1
         alphavalue = fixalpha
      endif else begin
         parinfo[4].limited[0] = 1
         parinfo[4].limited[1] = 1
         parinfo[4].limits[0] = 0.5
         parinfo[4].limits[1] = 3.0
         alphavalue = 2.0
         if shortflag lt 1 then begin 
            parinfo[4].fixed = 1
         endif
      endelse
      p0 = 4.0        ;best starting guess for N_bb
      p1 = 30         ;best starting guess for Tdust
      p2 = betavalue  ;best starting guess for beta
      p4 = alphavalue ;best starting guess for alpha
      npl = p0 + alog10(1-exp(-(200.*((26.6764 + 6.24629 * p4)^(-2.) + (0.000190530 + 7.24277e-05 * p4)*p1))^p2)) + 3.*alog10((26.6764 + 6.24629 * p4)^(-2.) + (0.000190530 + 7.24277e-05 * p4)*p1) - alog10(exp(14393.9*((26.6764 + 6.24629 * p4)^(-2.) + (0.000190530 + 7.24277e-05 * p4)*p1)/(p1))-1.)
      waveturn = ((26.6764 + 6.24629 * p4)^(-2.) + (0.000190530 + 7.24277e-05 * p4)*p1)^(-1.)
      parinfo[*].value = [p0, p1, betavalue, npl, alphavalue, waveturn]
      p0 = parinfo.value
      expr ='7.+4. + alog10( (10^P[0])*((1-exp(-(200./x)^(P[2])))*x^(-3.)/(exp(14393.9/(x*P[1]))-1.)) + (10^P[3])*(exp(-(x/(P[5]*3/4.))^2.)) * (x/P[5])^(P[4]) )'
      P = mpfitexpr(expr,xarr,yarr,yerr,p0,perror=dp,/quiet,parinfo=parinfo)
      fluxarr =7.+4. + alog10( (10^P[0])*((1-exp(-(200./wavearr)^(P[2])))*wavearr^(-3.)/(exp(14393.9/(wavearr*P[1]))-1.)) + (10^p[3])*(exp(-(wavearr/(p[5]*3./4.))^2.)) * (wavearr/p[5])^(p[4]) )
      fluxarr_bb =7.+4. + alog10( (10^P[0])*((1-exp(-(200./wavearr)^(P[2])))*wavearr^(-3.)/(exp(14393.9/(wavearr*P[1]))-1.)) )
      fluxarr_pl =7.+4. + alog10( (10^p[3])*(exp(-(wavearr/(p[5]*3./4.))^2.)) * (wavearr/p[5])^(p[4]) )
endif



; TEST TO SEE IF THE FIT WORKED/CONVERGED:
if size(dp,/n_dimensions) eq 0  then begin
            ; if for some reason the fit fails
            ; (usually because of wacky
            ; photometry) then return -1 and quit.
   print, "WCMCSED: MPFIT FAILED..."
   return, -1
endif

; ----------------------------- ;
; OUTPUT FIT PARAMETERS TO USER:
; ----------------------------- ;
z = redshift
sedwave = alog10(wavearr)
sedflux = fluxarr ;already on a log scale
sfpl = alog10(10^fluxarr-10^fluxarr_bb)
sfbb = fluxarr_bb
lfirtotal = sedtolfir(sedwave,sedflux,8.,1000.,redshift=z)
lfirlong = sedtolfir(sedwave,sedflux,40.,1000.,redshift=z)
lfir = lfirtotal
tdust_fit = p[1]
td_fit = tdust_fit
beta = p[2]
alpha = p[4]
; dust temperature is reset to the inverse peak wavelength of the SED:
tmp = max(sedflux,imaxsed)
lpeakarr = 10^sedwave[imaxsed]
tdust = 2.898e3/lpeakarr

; ----------------------------------------------------------
; --- DETERMINE UNCERTAINTIES BY FIXING OTHER PARAMETERS ---
; ----------------------------------------------------------

; ERROR IN LFIR:
lfirerr = sedtolfir(sedwave,sedflux+dp[0],8.,1000.,redshift=z) - lfirtotal
lerr = lfirerr

; ERROR IN TDUST (cold):
tdusterr = dp[1]
tderr = tdusterr

; ERROR IN BETA:
if keyword_set(fixbeta) eq 0 then begin 
   betaerr = dp[2] 
endif else begin
   betaerr= -99
endelse
berr = betaerr

; ERROR IN ALPHA:
if keyword_set(fixalpha) eq 0 then begin
   alphaerr = dp[4] 
endif else begin 
   alphaerr = -99
endelse
aerr = alphaerr

; ---------------------------
; --- CALCULATE DUST MASS ---
; ---------------------------
; (note this assumes a dust mass coefficient, kappa of kappa=0.15+/-0.02 m^2 kg^-1 at 850um)

tmp = min(abs(sedwave-alog10(850.)),imin850)
fit_s850 = 10^sedflux[imin850]

mdust = alog10(fit_s850) - 3. - 26. + 2.*alog10(lumdist(z,/cm))-4. - alog10(0.15) - alog10(planck(8.5e8,tdust_fit)) - alog10(1.988) - 30.
; using the dust temperature from the SED fit - not the peak dust
;                                               temperature - makes a
;                                                             difference (see paper Casey 2012)

; approximate the dust mass using the uncertainty in fit_s850, which is
; equal to the uncertainty in the normalization of the cold dust
; greybody (dp[0]) combined with uncertainty in kappa

mdusterr = alog10(2.71828183) * sqrt( (dp[0]/alog10(fit_s850))^2. + (0.02/0.15)^2. )
mderr = mdusterr

; --------------------------------------------------------
; --- ADD RADIO SYNCHROTRON COMPONENT IF KEYWORD SET ---
; --------------------------------------------------------
; (note has dependency on normalizepoint.pro function)
if keyword_set(radio) then begin
   synchalpha = 0.75
   radioflux = lfirtoradio(lfirtotal,redshift)
   radiosed = sedwave*synchalpha + normalizepoint(alog10(214286./(1.+redshift)),alog10(radioflux)-3.,sedwave,sedwave*synchalpha) 
   totalsed = alog10(10^sedflux + 10^radiosed)
   sedflux = totalsed
endif

;---------------------------------------------------------------
;        --- PLOT RESULTING FIT IN INTERACTIVE MODE ---
;---------------------------------------------------------------
if keyword_set(interactive) then begin
   window,1,xsize=800,ysize=600
   djs_plot,sedwave,sedflux, color='blue', $
            yran=[-2,(max([yarrall,sedflux])+0.7)<10.], xran=[0.5,5.], $
            xtitle="log(rest wavelength) [\mum]", ytitle="log(flux density) [mJy]"
   djs_oplot,sedwave,fluxarr_bb, color='light blue'
   djs_oplot,sedwave,fluxarr_pl, color='cyan'
   ver,p[5],line=3,color='cyan'
   djs_oplot,alog10(xarrall),yarrall,psym=1,color='red'
   oplot_lines,alog10(xarrall),yarrall+yerrall,alog10(xarrall),yarrall-yerrall,color='red'
   ver, alog10([8.,1000.]),color='magenta',thick=3
   ver, alog10([40.,1000.]),color='pink',thick=3,line=2
   legend, textoidl(['logL_{FIR(8-1000)} = '+strtrim(string(lfirtotal),1)+'\pm'+strtrim(string(lfirerr),1), $ ;'logL_{FIR(40-1000)} = '+strtrim(string(lfirlong),1), $
                     'T_{d} = '+strtrim(string(tdust),1)+'\pm'+strtrim(string(tdusterr),1)+'K', $
                     'logM_{d} = '+strtrim(string(mdust),1)+'\pm'+strtrim(string(mdusterr),1)+' M_{'+sunsymbol()+'}', $
                     '\beta = '+strtrim(string(beta),1)+'\pm'+strtrim(string(betaerr),1), $
                     '\alpha = '+strtrim(string(alpha),1)+'\pm'+strtrim(string(alphaerr),1), $
                     'z ='+string(redshift)]),charsize=1.4,/right
;   legend,[string(p)],/bottom,charsize=1.8
   print, "CLICK THE CURSOR ANYWHERE IN THE PLOT TO EXIT."
   cursor,x,y,/up
endif

; -----------------------------
; --- COMPUTE FIT RESIDUALS ---
; -----------------------------
; (data minus fit in log flux density)
residuals = xarrall*0.
for i=0, n_elements(residuals) - 1 do begin
   tmp = min(abs(alog10(xarrall[i])-sedwave),imin)
   residuals[i] = yarrall[i] - sedflux[imin]
endfor

lfir = lfirtotal

; RETURN THE SEDFLUX ARRAY
return, sedflux 

END
