;+
; NAME:
;         CMCIRSED_NOPL
; PURPOSE:
;         Fit an infrared (8-1000um) spectral energy distribution to
;         a galaxy's infrared data points - Operates just like
;                                           CMCIRSED but excludes the
;                                           powerlaw component in the
;                                           mid-IR.  See cmcirsed.pro
;                                           readme for more information.

FUNCTION cmcirsed_nopl, $
   DATAWAVE, DATAFLUX, REDSHIFT, FLUXERR=FLUXERR, $ ;required information
   FIXBETA=FIXBETA, RADIO=RADIO, $ ;optional inputs
   SEDWAVE=SEDWAVE, TDUST=TDUST, MDUST=MDUST, LFIR=LFIR, $ ;optional outputs
   BETA=BETA, TDERR=TDERR, MDERR=MDERR, LERR=LERR, BERR=BERR, RESIDUALS=RESIDUALS, $  ;optional outputs
   GENERALOPACITY=GENERALOPACITY, INTERACTIVE=INTERACTIVE ;optional tags


if n_params() eq 0 then begin
   print, "CMCIRSED_NOPL.PRO - fit a coupled greybody+powerlaw to FIR data"
   print, " SEDFLUX = CMCIRSED_NOPL( DATAWAVE, DATAFLUX, REDSHIFT, [FLUXERR=FLUXERR, $ "
   print, "                      FIXBETA=FIXBETA, $  ;(optional inputs) "
   print, "                      SEDWAVE=SEDWAVE, TDUST=TDUST, MDUST=MDUST, LFIR=LFIR, BETA=BETA, $ ;(optional outputs) "
   print, "                      TDERR=TDERR, MDERR=MDERR, LERR=LERR, BERR=BERR, AERR=AERR, RESIDUALS=RESIDUALS, $ ;(optional outputs) "
   print, "                      /GENERALOPACITY, /INTERACTIVE, /RADIO) ;(optional tags)"
   print, "                         input units: datawave - wavelength array in um (observed)"
   print, "                                      dataflux - flux density array in mJy (observed)"
   print, "                         output units: sedflux - log(flux density in mJy) (rest-frame)"
   print, "                                       sedwave - log(wavelength in um) (rest-frame)"
   print, "                                       tdust - dust temperature in K"
   print, "                                       lfir - log(Lfir) in solar luminosities"
   print, "                                       beta - emissivity"
   print, "                                       mdust - dust mass in log solar masses (from extrapolated 850um flux density)"
   print, "                         optional tags: fixbeta=betavalue - fix the emissivity to e.g. 1.5 or 2.0"
   print, "                                        /radio - add a radio synchrotron component, assumes FIR/radio correlation and scales to LFIR and slope of 0.8"
   print, "                                        /generalopacity - don't assume an optically thin BB"
   print, "                                        /interactive - plots up data and prompts user to select only certain FIR data points"
   print, "          ** modified version of cmcirsed.pro - 9 Mar 2012 ** see cmcirsed.pro for more information ** "
   return, -1
endif

; DEFINE SED VARIABLES:
; --- WAVELENGTH ---
wavearr = 10^makehx(0.5,5.4,0.001)
; in units of microns
; defining a wavelength array which runs from ~3.16um to
; 251189um~2.5cm rest-frame

; ANALYZE DATA FOR COMPLETENESS:
; --- DOES AN ERROR ARRAY NOT EXIST? ---
if keyword_set(fluxerr) eq 0 then begin
; set some realistic flux errors considering the wavelength regimes
; and typical telescopes used for the observations (i.e. confusion noise)...
   fluxerr= dataflux*0.
   tmp = where(datawave le 120.,ct)
   if ct gt 0 then fluxerr[tmp] = 1.
   tmp = where(datawave gt 120. and datawave le 360.,ct)
   if ct gt 0 then fluxerr[tmp] = 5.
   tmp = where(datawave gt 360. and datawave le 510.,ct)
   if ct gt 0 then fluxerr[tmp] = 9.
   tmp = where(datawave gt 510. and datawave le 890.,ct)
   if ct gt 0 then fluxerr[tmp] = 2.
   tmp = where(datawave gt 890.,ct)
   if ct gt 0 then fluxerr[tmp] = 1.
;   tmp = where(fluxerr_alternate-fluxerr gt 0)
;   fluxerr[tmp] = fluxerr_alternate[tmp]
   print, "***** WARNING - FLUXERR ARRAY NOT SPECIFIED, USING SOME FAKE DEFAULT VALUES!! *****" 
endif
gooddata = where(finite(dataflux) and finite(fluxerr))
datawave = datawave[gooddata]
dataflux = dataflux[gooddata]
fluxerr = fluxerr[gooddata]

ndata = n_elements(datawave)
xarr = datawave/(1.+redshift) ;xarr = rest wavelength
yarr = alog10(dataflux>0.001) ;yarr = log dataflux
yerr = yarr*0. ;yerr will be the uncertainty in yarr
;CHECK FOR NON-DETECTIONS *AND* COMPUTE LOG-ERR ARRAY=YERR:
for i=0, ndata-1, 1 do begin
   snr = dataflux[i]/fluxerr[i] 
   if snr le 1 then begin
      yerr[i] = alog10(fluxerr[i]+dataflux[i]>0.01)-alog10(dataflux[i]>0.01)
   endif else begin
      yerr[i] = 0.5*(alog10(fluxerr[i]+dataflux[i])-alog10(dataflux[i]-fluxerr[i]))
   endelse
endfor
xarrall=xarr
yarrall=yarr
yerrall=yerr


longwave = where(xarr ge 40.,ctl)
shortwave = where(xarr lt 50. and xarr ge 8.,cts)
if cts ge 1 then begin
   xshort = xarr[shortwave]
   yshort = yarr[shortwave]
   yshorterr = yerr[shortwave]
;;COMPARISON - COMPUTE OFFSET IN 8-25um RANGE
;   restore,'/Users/cmcasey/idl/basic/smg_sed_Pope08.sav'
;   smgwave = alog10(wave)
;   smgflux = alog10(lnu) - 25.
;   tmp = min(abs(smgwave - alog10(xshort[0])),imin)
;   shortoffset = yshort[0]-smgflux[imin]
endif

;MAKE SURE ALL DATA IS FINITE:
tmp = where(finite(yarr) and finite(yerr),ct)
if ct gt 0 then begin
   xarr = xarr[tmp]
   yarr = yarr[tmp]
   yerr = yerr[tmp]
endif else begin
   print, "FAIL - non finite data (e.g. data= NaN, Inf, or -Inf) - rerun with finite data flux points..."
   return, -1
endelse


;--------------------------------
; COMPUTE THE BEST FIT FIR SED:
;--------------------------------

fitcase = ''
if keyword_set(generalopacity) then fitcase='generalopacity' else fitcase='otthin'

shortflag = cts
if shortflag eq 1 then begin
   snrshort = 10^(yshort)/(10^(yshort+yshorterr)-10^(yshort))
   if snrshort le 5. then shortflag = 0.5
endif
longflag = n_elements(longwave)

;print, 'shortflag = ', shortflag
;print, 'longflag = ', shortflag

; DESCRIPTION OF FIT PARAMETERS -->
      ; the coefficients of the fit are as follows:
      ; [norm1, tdust, beta, (norm2), alpha, (waveturn)]
      ; norm1 is the normalization of the SED, proportional to LIR
      ; tdust is the dust temperature of the system,
      ;   representing the emission from the greybody
      ;   constrain dust temperature to 10<tdust<170K
      ; beta is the emissivity of the greybody
      ; norm2 is the normalization of the powerlaw component
      ;   and it is fixed to scale with norm1 and tdust
      ; alpha is the slope of the powerlaw component
      ; waveturn is the wavelength where the powerlaw turns over
      ;   and it is fixed to scale with tdust

; OPTICALLY THIN GREYBODY FIT
if fitcase eq 'otthin' then begin
      parinfo = replicate({value:0.D, fixed:0, tied:'', limited:[0,0],limits:[0.D,0]}, 3)
      parinfo[1].limited[0] = 1
      parinfo[1].limited[1] = 1
      parinfo[1].limits[0] = 10.
      parinfo[1].limits[1] = 170.
      ; fix beta if fixbeta is set
      if keyword_set(fixbeta) ne 0 then begin
         parinfo[2].fixed = 1
         betavalue = fixbeta
      endif else begin
         parinfo[2].limited[0] = 1
         parinfo[2].limited[1] = 1
         parinfo[2].limits[0] = 0.5
         parinfo[2].limits[1] = 3.0
         betavalue = 1.5
      endelse
      parinfo[*].value = [4.0, 30., betavalue] ; , -5.0, alphavalue, 35.]
      p0 = parinfo.value
      expr ='7.+4.+alog10( (10^P[0])*(x^(-(3.+p[2]))/(exp(14393.9/(x*P[1]))-1.)))' ; + (10^P[3]) * exp(-(x/p[5])^2.) * (x/p[5])^p[4] ) '
      P = mpfitexpr(expr,xarr,yarr,yerr,p0,perror=dp,/quiet,parinfo=parinfo)
      fluxarr =7.+4.+alog10( (10^P[0])*(wavearr^(-(3.+p[2]))/(exp(14393.9/(wavearr*P[1]))-1.))) ; + 10^p[3]*exp(-(wavearr/p[5])^2.) * (wavearr/p[5])^p[4])
;      fluxarr_bb =7.+4. + alog10( (10^P[0])*(wavearr^(-(3.+p[2]))/(exp(14393.9/(wavearr*P[1]))-1.)) )
;      fluxarr_pl =7.+4. + alog10( (10^p[3])*(exp(-(wavearr/p[5])^2.)) * (wavearr/p[5])^(p[4]) )
   endif

; GENERALOPACITY GREYBODY FIT
if fitcase eq 'generalopacity' then begin
      parinfo = replicate({value:0.D, fixed:0, tied:'', limited:[0,0],limits:[0.D,0]}, 3)
      ; [norm1, tdust, beta, (norm2), alpha, (waveturn)]
      parinfo[1].limited[0] = 1
      parinfo[1].limited[1] = 1
      parinfo[1].limits[0] = 10.
      parinfo[1].limits[1] = 170.
      ; fix beta if fixbeta is set
      if keyword_set(fixbeta) ne 0 then begin
         parinfo[2].fixed = 1
         betavalue = fixbeta
      endif else begin
         parinfo[2].limited[0] = 1
         parinfo[2].limited[1] = 1
         parinfo[2].limits[0] = 0.5
         parinfo[2].limits[1] = 3.0
         betavalue = 1.5
      endelse
      parinfo[*].value = [4.0, 30., betavalue] ;, -5.0, alphavalue, 35.]
      p0 = parinfo.value
      expr ='7.+4. + alog10( (10^P[0])*((1-exp(-(200./x)^(P[2])))*x^(-3.)/(exp(14393.9/(x*P[1]))-1.)))' ; + (10^P[3])*(exp(-(x/P[5])^2.)) * (x/P[5])^(P[4]) )'
      P = mpfitexpr(expr,xarr,yarr,yerr,p0,perror=dp,/quiet,parinfo=parinfo)
      print, p
      fluxarr =7.+4. + alog10( (10^P[0])*((1-exp(-(200./wavearr)^(P[2])))*wavearr^(-3.)/(exp(14393.9/(wavearr*P[1]))-1.)))
 ; + (10^p[3])*(exp(-(wavearr/p[5])^2.)) * (wavearr/p[5])^(p[4]) )
;      fluxarr_bb =7.+4. + alog10( (10^P[0])*((1-exp(-(200./wavearr)^(P[2])))*wavearr^(-3.)/(exp(14393.9/(wavearr*P[1]))-1.)) )
;      fluxarr_pl =7.+4. + alog10( (10^p[3])*(exp(-(wavearr/p[5])^2.)) * (wavearr/p[5])^(p[4]) )
endif


; TEST TO SEE IF THE FIT WORKED/CONVERGED:
if size(dp,/n_dimensions) eq 0  then begin
            ; if for some reason the fit fails
            ; (usually because of wacky
            ; photometry) then return -1 and quit.
   print, "WCMCSED: MPFIT FAILED..."
   return, -1
endif

; ----------------------------- ;
; OUTPUT FIT PARAMETERS TO USER:
; ----------------------------- ;
z = redshift
sedwave = alog10(wavearr)
sedflux = fluxarr ;already on a log scale
;sfbb = fluxarr_bb
lfirtotal = sedtolfir(sedwave,sedflux,8.,1000.,redshift=z)
lfirlong = sedtolfir(sedwave,sedflux,40.,1000.,redshift=z)
tdust_fit = p[1]
beta = p[2]
; dust temperature is reset to the inverse peak wavelength of the SED:
tmp = max(sedflux,imaxsed)
lpeakarr = 10^sedwave[imaxsed]
tdust = 2.898e3/lpeakarr

; ----------------------------------------------------------
; --- DETERMINE UNCERTAINTIES BY FIXING OTHER PARAMETERS ---
; ----------------------------------------------------------

; ERROR IN LFIR:
lfirerr = sedtolfir(sedwave,sedflux+dp[0],8.,1000.,redshift=z) - lfirtotal

; ERROR IN TDUST (cold):
tdusterr = dp[1]

; ERROR IN BETA:
if keyword_set(fixbeta) eq 0 then begin 
   betaerr = dp[2] 
endif else begin
   betaerr= -99
endelse

; ---------------------------
; --- CALCULATE DUST MASS ---
; ---------------------------
; (note this assumes a dust mass coefficient, kappa of kappa=0.15+/-0.02 m^2 kg^-1 at 850um)

tmp = min(abs(sedwave-alog10(850.)),imin850)
fit_s850 = 10^sedflux[imin850]

mdust = alog10(fit_s850) - 3. - 26. + 2.*alog10(lumdist(z,/cm))-4. - alog10(0.15) - alog10(planck(8.5e8,tdust_fit)) - alog10(1.988) - 30.

; approximate the dust mass using the uncertainty in fit_s850, which is
; equal to the uncertainty in the normalization of the cold dust
; greybody (dp[0]) combined with uncertainty in kappa

mdusterr = alog10(2.71828183) * sqrt( (dp[0]/alog10(fit_s850))^2. + (0.02/0.15)^2. )

; --------------------------------------------------------
; --- ADD RADIO SYNCHROTRON COMPONENT IF KEYWORD SET ---
; --------------------------------------------------------
; (note has dependency on normalizepoint.pro function)
if keyword_set(radio) then begin
   synchalpha = 0.80
   radioflux = lfirtoradio(lfirtotal,redshift)
   radiosed = sedwave*synchalpha + normalizepoint(alog10(214286./(1.+redshift)),alog10(radioflux)-3.,sedwave,sedwave*synchalpha) 
   totalsed = alog10(10^sedflux + 10^radiosed)
   sedflux = totalsed
endif

;---------------------------------------------------------------
;        --- PLOT RESULTING FIT IN INTERACTIVE MODE ---
;---------------------------------------------------------------
if keyword_set(interactive) then begin
   window,1,xsize=800,ysize=600
   djs_plot,sedwave,sedflux, color='blue', $
            yran=[-2,(max([yarrall,sedflux])+0.7)<10.], xran=[0.5,5.], $
            xtitle="log(rest wavelength) [\mum]", ytitle="log(flux density) [mJy]"
;   djs_oplot,sedwave,fluxarr_pl, color='cyan'
;   ver,p[5],line=3,color='cyan'
   djs_oplot,alog10(xarrall),yarrall,psym=1,color='red'
   oplot_lines,alog10(xarrall),yarrall+yerrall,alog10(xarrall),yarrall-yerrall,color='red'
   ver, alog10([8.,1000.]),color='magenta',thick=3
   ver, alog10([40.,1000.]),color='pink',thick=3,line=2
   legend, textoidl(['logL_{FIR(8-1000)} = '+strtrim(string(lfirtotal),1)+'\pm'+strtrim(string(lfirerr),1), $ ;'logL_{FIR(40-1000)} = '+strtrim(string(lfirlong),1), $
                     'T_{d} = '+strtrim(string(tdust),1)+'\pm'+strtrim(string(tdusterr),1)+'K', $
                     'logM_{d} = '+strtrim(string(mdust),1)+'\pm'+strtrim(string(mdusterr),1)+' M_{'+sunsymbol()+'}', $
                     '\beta = '+strtrim(string(beta),1)+'\pm'+strtrim(string(betaerr),1), $
                     'z ='+string(redshift)]),charsize=1.4,/right
;   legend,[string(p)],/bottom,charsize=1.8
   print, "CLICK THE CURSOR ANYWHERE IN THE PLOT TO EXIT."
   cursor,x,y,/up
endif

; -----------------------------
; --- COMPUTE FIT RESIDUALS ---
; -----------------------------
; (data minus fit in log flux density)
residuals = xarrall*0.
for i=0, n_elements(residuals) - 1 do begin
   tmp = min(abs(alog10(xarrall[i])-sedwave),imin)
   residuals[i] = yarrall[i] - sedflux[imin]
endfor

lfir = lfirtotal

; RETURN THE SEDFLUX ARRAY
return, sedflux 

END
