;+
; NAME:
;         TDTOSED
; PURPOSE:
;         Generate a FIR SED for a galaxy of certain dust temperature

; CALLING SEQUENCE:
;      sedflux = tdtosed(T_D, redshift=redshift, [, logfir=logfir, $
;                radioflux=radioflux, sedwave=sedwave,beta=beta, $
;                \generalopacity, \nopowerlaw, sedfluxbb=sedfluxbb])
;
; INPUT:
;      T_D - dust temperature of the cold component fit (note
;            this isn't where the dust SED peaks
;      redshift - the galaxy's redshift (everything is in rest-frame,
;                 but this is used for the absolute flux density scale
;                 with given logfir or radioflux)
;      logfir - FIR luminosity (8-1000um) to determine the absolute
;               scaling
;      radioflux - this can be given in place of logfir, then logfir
;                  is computed using FIR radio correlation
;      beta - emissivity.  if not specified, default is 1.5
;
; OUTPUT:
;      sedflux - SED in units of log mJy
;      sedwave - wavelength array for sedflux, in log um
;      sedfluxbb - SED without mid-infrared powerlaw
;
; OPTIONAL TAGS:
;     \generalopacity - use generalopacity instead of optically thin
;                       model
;     \nopowerlaw - do not include the mid-infrared powerlaw
;                   component (as described in more detail in
;                   cmcirsed.pro)
;     
;
; Revision History:
;    Written originally in 2008 by Caitlin M. Casey
;    Revised a few times... released March 2012   
;
FUNCTION TDTOSED, TD, SEDWAVE=SEDWAVE, REDSHIFT=REDSHIFT, RADIOFLUX=RADIOFLUX, LOGFIR=LOGFIR, BETA=BETA, ALPHA=ALPHA, GENERALOPACITY=GENERALOPACITY, NOPOWERLAW=NOPOWERLAW, SEDFLUXBB=SEDFLUXBB, MDUST=MDUST, TDUST=TDUST

if n_params() eq 0 then begin
    print, 'tdtosed.pro '
    print, '        USAGE: SEDFLUX = TDTOSED(T_D, redshift=redshift, [,radioflux=radioflux, logfir=logfir, sedwave=sedwave, beta=beta, alpha=alpha, \generalopacity, \nopowerlaw, sedfluxbb=sedfluxbb, mdust=mdust, tdust=tdust])'
    print, '                NOTE - either RADIOFLUX or LOGFIR must be given to normalize the SED'
    print, '        INPUT:  Td (single value), redshift (single value), either radioflux or logfir, beta'
    print, '        OUTPUT: SEDFLUX in log of f_lambda in mJy, SEDWAVE (optional output) given as log of wavelength in um'
    print, '                SEDFLUXBB is also optional output: it is the flux of just the sedfit without the powerlaw added on'
    print, '                  \generalopacity - use a general opacity fit rather than the optically thin approximation'
    print, '                  \nopowerlaw - do not add the mid-infrared powerlaw component (which is added by default)'
    print, '                  mdust/tdust - output the dust mass and new characteristic dust temperature (1/lambda_peak)'
    print, '   written 2008 - cmc'
    return, -1
endif

if keyword_set(radioflux) ne 0 then begin
    logintS = radiotolfir(radioflux,redshift) - alog10(4.*!pi) - 2.*alog10(lumdist(redshift,/cm,/silent,h0=71,lambda0=0.73)) + alog10(3.8) + 33.
    logfir = radiotolfir(radioflux,redshift)
endif

if (keyword_set(logfir) ne 0 and logfir ge 5.) then begin
;the integrated S_nu function is this:
    logintS = logfir - alog10(4.*!pi) - 2.*alog10(lumdist(redshift,/cm,/silent,h0=71,lambda0=0.73)) + alog10(3.8) + 33.
endif ;else begin
;    logfir = radiotolfir(radioflux,redshift)
;endelse

if keyword_set(beta) ne 0 then begin
    beta = beta
endif else begin
    beta = 1.5
endelse

                                                 
if keyword_set(alpha) ne 0 then begin
   alpha = alpha
endif else begin 
   alpha = 2.0
endelse

wavearr = 10^makehx(0.5,5.4,0.001)

nuarr = 3.e8/(wavearr*10^(-6.)) ;frequency
nubin = nuarr[1:*] - nuarr[0:n_elements(nuarr)-1]
nubin = [nubin,-1] 
nubin[n_elements(nuarr)-1] = nubin[n_elements(nuarr)-2]

   hoverk=alog10(6.626)-34 - alog10(1.381)+23. ; h (planck const) divided by k (boltzmann const)
   arg = dblarr(n_elements(wavearr))
   logS_nu = dblarr(n_elements(wavearr))
   arg = (exp((10^hoverk*nuarr)/(td))-1.)^(-1.)

if keyword_set(generalopacity) eq 0 then begin 
   logS_nu = (3.+beta)*alog10(nuarr) + alog10(arg)
endif else begin 
   nu0 = 3.e8/(200. * 1.e-6)
   tau = (nuarr/nu0)^beta
   logS_nu = alog10(1-exp(-tau)) + 3.*alog10(nuarr) +  alog10(arg)
endelse

if keyword_set(nopowerlaw) eq 0 then begin 
   tmp = max(logs_nu,imax)
   tx = alog10(wavearr[imax])
   ty = logs_nu[imax]
;powerlaw component:
   if keyword_set(generalopacity) eq 0 then begin
      p0 = ty + alog10( (exp(14393.9/(10^tx*td))-1)) + (3+beta)*tx 
      ;optically thin:
      critwave = (1./td)*(0.000310933 + 6.99836e-05*alpha)^(-1.)
      offset = p0 + (3 + beta)*alog10(td * (0.000310933 + 6.99836e-05*alpha)) - alog10(exp(14393.9 * (0.000310933 + 6.99836e-05 * alpha ))-1)
   endif else begin 
      ;general opacity:
      p0 = ty + alog10( (exp(14393.9/(10^tx*td))-1)) - alog10(1-exp(-(200./10^tx)^beta)) + 3.*tx
      critwave = ((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*td)^(-1.)
      offset = p0 + alog10(1-exp(-(200.*((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*td))^beta)) + 3.*alog10((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*td) - alog10(exp(14393.9*((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*td)/(td))-1.)
   endelse
;   critwave = 1./(5.08308e-6 + 0.000927256 * td)
   
   logS_nu_pl = alog10(exp(-(wavearr/(critwave*0.75))^2.)) + alpha*alog10(wavearr/critwave) + offset
 
;   logS_nu_pl = logS_nu_pl + (max(logS_nu) - max(logS_nu_pl) - 0.8)
   
   const2 = alog10(2.*6.626e-34) - alog10(3.^2.)-8.*2.
   logS_nu = logS_nu + const2
   logS_nu_pl = logS_nu_pl + const2
;(log)S_nu has units of W m^-2.  Convert to erg cm^-2 s^-1 --> 10^7 for energy, (100.)^-2=10^-4 for distance
   logS_nu = logS_nu + 7. - 4.
   logS_nu_pl = logS_nu_pl + 7. - 4.
;(log)S_nu now has units of erg cm^-2 s^-1
   S_nu = 10^logS_nu
   S_nu_pl = 10^logS_nu_pl

   logS_nu_tot = alog10(S_nu+S_nu_pl)
endif else begin 
   const2 = alog10(2.*6.626e-34) - alog10(3.^2.)-8.*2.
   logS_nu = logS_nu + const2
;(log)S_nu has units of W m^-2.  Convert to erg cm^-2 s^-1 --> 10^7 for energy, (100.)^-2=10^-4 for distance
   logS_nu = logS_nu + 7. - 4.
   logS_nu_tot = logS_nu
   s_nu = 10^logs_nu
endelse


bandwidth = where(wavearr ge 8. and wavearr le 1000.) ;integrate lfir over these wavelengths

intS = alog10(-total(reform(S_nu[bandwidth])*nubin[bandwidth]/(1+redshift))) ;/nuarr[bandwidth]))
sedflux = logs_nu_tot + logintS - intS+26.
sedfluxbb = logs_nu + logintS - intS+26.
sedwave = alog10(wavearr)

;correct for luminosity excess in mid-infrared:

lfir1 = sedtolfir(sedwave,sedflux,8,1000,redshift=redshift)
sedflux = sedflux - (lfir1-logfir)
sedfluxbb = sedfluxbb - (lfir1-logfir)

;if keyword_set(radioflux) eq 0 then radioflux = lfirtoradio(logfir,redshift)
;radio = sedwave*0.8+normalizepoint(alog10(214286./(1.+redshift)),alog10(radioflux)-3.,sedwave,sedwave*0.8)
;sedflux = alog10(10^sedflux+10^radio)

tmp = min(abs(sedwave-alog10(850.)),imin850)
fit_s850 = 10^sedflux[imin850]
mdust = alog10(fit_s850) - 3. - 26. + 2.*alog10(lumdist(redshift,/cm))-4. - alog10(0.15) - alog10(planck(8.5e8,td)) - alog10(1.988) - 30.

tmp = max(sedflux,imax)
tdust = 2.898e3/(10^sedwave[imax])

return, sedflux

END

